import * as Path from "path";
import { Tree } from "@himenon/path-oriented-data-structure";
import Dot from "dot-prop";
import { UnSupportError } from "../../Exception";
import * as Def from "./Definition";
import * as Operation from "./Operation";
import * as State from "./State";
import * as Structure from "./structure";
class Store {
    constructor(factory, rootDocument) {
        this.factory = factory;
        this.state = State.createDefaultState(rootDocument);
        const { operator, getChildByPaths } = Structure.create();
        this.operator = operator;
        this.getChildByPaths = getChildByPaths;
    }
    convertNamespace(tree) {
        const statements = [];
        Object.values(tree.getChildren()).map(child => {
            if (child instanceof Tree || child instanceof Structure.NamespaceTree.Item) {
                statements.push(this.convertNamespace(child));
            }
            else if (child instanceof Structure.InterfaceNode.Item) {
                statements.push(child.value);
            }
            else if (child instanceof Structure.TypeAliasNode.Item) {
                statements.push(child.value);
            }
        });
        if (tree instanceof Structure.NamespaceTree.Item) {
            return this.factory.Namespace.create({
                export: true,
                name: tree.params.name,
                statements,
                comment: tree.params.comment,
                deprecated: tree.params.deprecated,
            });
        }
        return this.factory.Namespace.create({
            export: true,
            name: tree.name,
            statements,
        });
    }
    capitalizeFirstLetter(text) {
        return text.charAt(0).toUpperCase() + text.slice(1);
    }
    getRootStatements() {
        // Debug Point: 抽象的なデータ構造全体を把握するために出力すると良い
        // fs.writeFileSync("debug/tree.json", JSON.stringify(this.operator.getHierarchy(), null, 2), { encoding: "utf-8" });
        const statements = Def.componentNames.reduce((statements, componentName) => {
            const treeOfNamespace = this.getChildByPaths(componentName, "namespace");
            if (treeOfNamespace) {
                treeOfNamespace.name = this.capitalizeFirstLetter(treeOfNamespace.name);
                return statements.concat(this.convertNamespace(treeOfNamespace));
            }
            return statements;
        }, []);
        return statements;
    }
    getAdditionalStatements() {
        return this.state.additionalStatements;
    }
    /**
     * @params path: "components/headers/hoge"
     */
    hasStatement(path, types) {
        const alreadyRegistered = types.some(type => !!this.operator.getChildByPaths(path, type));
        return alreadyRegistered;
    }
    /**
     * @params path: "components/headers/hoge"
     */
    addStatement(path, statement, options) {
        if (!path.startsWith("components")) {
            throw new UnSupportError(`componentsから始まっていません。path=${path}`);
        }
        const targetPath = Path.posix.relative("components", path);
        // すでにinterfaceまたはNAMESPACEとして登録がある場合はスキップ
        if (this.hasStatement(targetPath, ["interface", "namespace"])) {
            return;
        }
        // もしTypeAliasが同じスコープに登録されているかつ、interfaceが新しく追加しようとしている場合、既存のstatementを削除する
        if (!!(options === null || options === void 0 ? void 0 : options.override) || (this.hasStatement(targetPath, ["typeAlias"]) && statement.kind === "interface")) {
            this.operator.remove(targetPath, "typeAlias");
        }
        this.operator.set(targetPath, Structure.createInstance(statement));
    }
    getStatement(path, kind) {
        const targetPath = Path.posix.relative("components", path);
        // components/schemasの場合
        if (path.split("/").length === 2 && kind === "namespace") {
            const child = this.getChildByPaths(targetPath, kind);
            if (child) {
                // FIXME Side Effect
                child.name = this.capitalizeFirstLetter(child.name);
            }
            return child;
        }
        return this.getChildByPaths(targetPath, kind);
    }
    addComponent(componentName, statement) {
        this.operator.set(`${componentName}`, Structure.createInstance(statement));
    }
    getNoReferenceOperationState() {
        return Operation.create(this.state.document);
    }
    updateOperationState(httpMethod, requestUri, operationId, newOperationState) {
        let operationState = this.state.operations[operationId];
        if (operationState) {
            operationState = { ...operationState, ...newOperationState };
        }
        else {
            operationState = State.createDefaultOperationState(httpMethod, requestUri, newOperationState);
        }
        this.state.operations[operationId] = operationState;
    }
    addAdditionalStatement(statements) {
        this.state.additionalStatements = this.state.additionalStatements.concat(statements);
    }
    getPathItem(localPath) {
        if (!localPath.startsWith("components/pathItem")) {
            throw new Error("Only use start with 'component/pathItems': " + localPath);
        }
        const result = Dot.get(this.state.document, localPath.replace(/\//g, "."));
        if (!result) {
            throw new Error(`Not found ${localPath}`);
        }
        return result;
    }
    getParameter(localPath) {
        if (!localPath.startsWith("components/parameters")) {
            throw new Error("Only use start with 'component/parameters': " + localPath);
        }
        const result = Dot.get(this.state.document, localPath.replace(/\//g, "."));
        if (!result) {
            throw new Error(`Not found ${localPath}`);
        }
        return result;
    }
    isAfterDefined(referencePath) {
        return !!Dot.get(this.state.document, referencePath.replace(/\//g, "."));
    }
}
export { Store };
//# sourceMappingURL=Store.js.map