import { posix as path } from "path";
import { DevelopmentError, FeatureDevelopmentError, NotFoundFileError } from "../../Exception";
import { FileSystem } from "../../FileSystem";
import * as Logger from "../../Logger";
import * as Guard from "../Guard";
const localReferencePatterns = [
    "#/components/schemas/",
    "#/components/responses/",
    "#/components/parameters/",
    "#/components/examples/",
    "#/components/requestBodies/",
    "#/components/headers/",
    "#/components/securitySchemes/",
    "#/components/links/",
    "#/components/callbacks/",
    "#/components/pathItems/",
];
export const localReferenceComponents = {
    "#/components/schemas/": "components/schemas",
    "#/components/responses/": "components/responses",
    "#/components/parameters/": "components/parameters",
    "#/components/examples/": "components/examples",
    "#/components/requestBodies/": "components/requestBodies",
    "#/components/headers/": "components/headers",
    "#/components/securitySchemes/": "components/securitySchemes",
    "#/components/links/": "components/links",
    "#/components/callbacks/": "components/callbacks",
    "#/components/pathItems/": "components/pathItems",
};
export const getLocalReferencePattern = (reference) => {
    let localReferencePattern;
    localReferencePatterns.forEach(referencePattern => {
        if (new RegExp("^" + referencePattern).test(reference.$ref)) {
            localReferencePattern = referencePattern;
        }
    });
    return localReferencePattern;
};
export const generateLocalReference = (reference) => {
    const localReferencePattern = getLocalReferencePattern(reference);
    if (!localReferencePattern) {
        return;
    }
    const name = reference.$ref.split(localReferencePattern)[1];
    const localPath = path.posix.join(localReferenceComponents[localReferencePattern], name);
    if (!localPath.startsWith("components")) {
        throw new DevelopmentError(`localPath is not start "components":\n${localPath}`);
    }
    return {
        type: "local",
        name,
        path: localPath,
    };
};
export const generateReferencePoint = (currentPoint, reference) => {
    const basedir = path.dirname(currentPoint);
    const ref = reference.$ref;
    const referencePoint = path.join(basedir, ref);
    return referencePoint;
};
export const generate = (entryPoint, currentPoint, reference) => {
    const localReference = generateLocalReference(reference);
    if (localReference) {
        return localReference;
    }
    if (reference.$ref.startsWith("http")) {
        throw new FeatureDevelopmentError("Please Pull Request ! Welcome !");
    }
    const referencePoint = generateReferencePoint(currentPoint, reference);
    if (!FileSystem.existSync(referencePoint)) {
        Logger.showFilePosition(entryPoint, currentPoint, referencePoint);
        Logger.error(JSON.stringify(reference, null, 2));
        throw new NotFoundFileError(`Not found reference point from current point. \n Path: ${referencePoint}`);
    }
    const fragmentIndex = referencePoint.indexOf("#/");
    let targetPath;
    if (fragmentIndex !== -1) {
        targetPath = referencePoint.substring(fragmentIndex + 2);
    }
    else {
        const relativePathFromEntryPoint = path.relative(path.dirname(entryPoint), referencePoint); // components/hoge/fuga.yml
        const pathArray = relativePathFromEntryPoint.split(path.sep); // ["components", "hoge", "fuga"]
        if (pathArray[0] !== "components") {
            throw new DevelopmentError(`targetPath is not start "components":\n${relativePathFromEntryPoint}`);
        }
        const ext = path.extname(relativePathFromEntryPoint); // .yml
        targetPath = pathArray.join("/").substring(0, relativePathFromEntryPoint.length - ext.length); // components/hoge/fuga
    }
    const pathArray = targetPath.split("/"); // ["components", "hoge", "fuga"]
    const schemaName = pathArray[pathArray.length - 1]; // fuga
    const componentName = pathArray[0] === "components" ? pathArray[1] : "";
    const data = FileSystem.loadJsonOrYaml(referencePoint);
    if (Guard.isReference(data)) {
        return generate(entryPoint, referencePoint, data);
    }
    return {
        type: "remote",
        referencePoint,
        path: targetPath,
        name: schemaName,
        componentName: Guard.isComponentName(componentName) ? componentName : undefined,
        data,
    };
};
export const resolveRemoteReference = (entryPoint, currentPoint, reference) => {
    if (reference.$ref.startsWith("#") || reference.$ref.startsWith("http")) {
        return { referencePoint: currentPoint, data: reference };
    }
    const referencePoint = generateReferencePoint(currentPoint, reference);
    if (!FileSystem.existSync(referencePoint)) {
        Logger.showFilePosition(entryPoint, currentPoint, referencePoint);
        Logger.error(JSON.stringify(reference, null, 2));
        throw new NotFoundFileError(`Not found reference point from current point. \n Path: ${referencePoint}`);
    }
    const data = FileSystem.loadJsonOrYaml(referencePoint);
    if (Guard.isReference(data)) {
        return resolveRemoteReference(entryPoint, referencePoint, data);
    }
    return {
        referencePoint,
        data,
    };
};
export const resolveLocalReference = (entryPoint, currentPoint, reference) => {
    if (!reference.$ref.startsWith("#")) {
        return reference;
    }
    const referencePoint = generateReferencePoint(currentPoint, reference);
    const data = FileSystem.loadJsonOrYaml(referencePoint);
    if (Guard.isReference(data)) {
        return resolveRemoteReference(entryPoint, referencePoint, data);
    }
    return data;
};
//# sourceMappingURL=Reference.js.map